	[org 0x7c00]
	[bits 16]
	BOOTLOADER_OFFSET equ 0x8000 ; Second stage of bootloader

	;; Clear some registers, useful for real computers
	mov ax, 0x0000
	mov ds, ax
	mov es, ax
	mov ss, ax

	mov al, 0x02
	mov [BOOT_DRIVE], dl ; Remember that the BIOS sets us the boot drive in 'dl' on boot
	mov bp, 0x7b00	 ; Stack grows downwards
	mov sp, bp

	mov bx, STARTED_LOADING
	call print
	call print_nl

	call load_bootloader ; read the kernel (and the rest of the bootloader) from disk

	mov bx, STARTING_STAGE
	call print
	call print_nl
	jmp BOOTLOADER_OFFSET
	jmp $ ; Should never be called

	%include "boot/disk.asm"
	%include "boot/print.asm"
	%include "boot/print_hex.asm"

load_bootloader:
	;mov dh, [SEC_COUNT]
	;call print_hex
	pusha
	
	call disk_load
	popa

	;Increment CURRENT_REG_LOAD
	mov cx, [CURRENT_REG_LOAD]
	add cx, 0x20
	mov [CURRENT_REG_LOAD], cx


	cmp al, 0x12 ; If al >= 0x12, increment cylinder. 18 = 0x12
	jge inc_head

	mov ch, [CYL_COUNT]
	cmp ch, 10
	jge done_load

	;Increment sector ID
	inc al
	jmp load_bootloader

inc_head:
	mov dh, [HEAD_COUNT]
	cmp dh, 1
	jge inc_cyl
	inc dh
	mov [HEAD_COUNT], dh
	mov al, 1
	jmp load_bootloader

inc_cyl:
	mov ch, [CYL_COUNT]
	inc ch
	mov [CYL_COUNT], ch
				;Reset head count
	mov dh, 0
	mov [HEAD_COUNT], dh
				;Reset sector count
	mov al, 1
	jmp load_bootloader

done_load:
	ret

	CYL_COUNT db 0
	HEAD_COUNT db 0
	CURRENT_REG_LOAD dw 0x800 ;Same as 0x8000, but is the register instead. Needs to be divisible by 0x100 so we don't go over a 64K boundary
	BOOT_DRIVE db 0 ; It is a good idea to store it in memory because 'dl' may get overwritten

	STARTED_LOADING db "Loading SteveOS...", 0
	STARTING_STAGE db "Starting bootloader second stage...", 0
	; padding
	times 510 - ($-$$) db 0
	dw 0xaa55
