	;; Sets up VESA video before going into 32 bit mode. Scans to see what modes are available, and selects the best one.

	
	bits 16

	;; vbe_set_mode:
	;; Sets a VESA mode
	;; In\	AX = Width
	;; In\	BX = Height
	;; In\	CL = Bits per pixel
	;; Out\	FLAGS = Carry clear on success
	;; ^^^^^Don't trust these! They're overwritten at the moment.^^^^^

vbe_set_mode:
	mov ax, 1280
	mov bx, 600
	mov cl, 24
	mov [width], ax
	mov [height], bx
	mov [bpp], cl
	
scan_modes:
	
	push es						;Apparently some VESA BIOSes destroy es
	mov ax, 0x4F00 				;Get VESA BIOS information
	mov di, vbe_info_block
	int 0x10
	pop es
	
	cmp ax, 0x004F 				;Test for error
	jne scan_mode_error
	
	mov ax, word[vbe_info_block + vbe_info_structure.video_modes] ; Offset
	mov [vbe_info_offset], ax
	mov ax, word[vbe_info_block + vbe_info_structure.video_modes + 2] ; Segment
	mov [vbe_info_segment], ax

	mov bx, vbe_info_block + vbe_info_structure.video_modes

	mov ax, [vbe_info_segment]
	mov fs, ax
	mov si, [vbe_info_offset]




	;//VBE_INFO equ 0x7e00

find_mode:
	mov dx, [fs:si]
	add si, 2
	mov [vbe_info_offset], si
	mov [mode], dx
	mov ax, 0
	mov fs, ax
 
	cmp dword [mode], 0xFFFF			; end of list?
	je find_error
 
	push es
	mov ax, 0
	mov es, ax
	mov ax, 0x4F01				; get VBE mode info
	mov cx, [mode]
	mov di, VBE_INFO
	int 0x10
	pop es
 
	cmp ax, 0x4F
	jne mode_info_error
 
	mov dx, [VBE_INFO + 0x19]
	call print_hex
	call print_nl

	mov ax, [width]
	cmp ax, [VBE_INFO + 0x12] 	;Width
	jg .next_mode
 
	mov ax, [height]
	cmp ax, [VBE_INFO + 0x14] 	;Height
	jg .next_mode
	
	mov al, [bpp]
	cmp al, [VBE_INFO + 0x19]	;BPP
	jg .next_mode
 
	; If we make it here, we've found the correct mode!
	 
	; Set the mode
	push es
	mov ax, 0x4F02
	mov bx, [mode]
	or bx, 0x4000			; enable LFB
	mov di, 0			; not sure if some BIOSes need this... anyway it doesn't hurt
	int 0x10
	pop es
 
	cmp ax, 0x4F
	jne find_error
 
	ret
 
.next_mode:
	mov ax, [vbe_info_segment]
	mov fs, ax
	mov si, [vbe_info_offset]
	jmp find_mode
 
find_error:
	mov bx, FIND_ERR_MSG
	call print
	jmp $

mode_info_error:
	mov bx, INFO_ERR_MSG
	call print
	jmp $


scan_mode_error:
	mov bx, ERROR_MSG
	call print
	jmp $



	ERROR_MSG db "ERR getting VESA info", 0
	FIND_ERR_MSG db "ERR finding proper VBE mode", 0
	INFO_ERR_MSG db "ERR getting mode info", 0

struc vbe_info_structure
	.signature:		resb 4
	.version:		resw 1
	.oem:			resd 1
	.capabilities:	resd 1
	.video_modes: 	resd 1
	.video_memory: 	resw 1
	.software_rev: 	resw 1
	.vendor:		resd 1
	.product_name:	resd 1
	.product_rev:	resd 1
	.reserved:		resb 222
	.oem_data:		resb 256
endstruc

vbe_info_block:
	istruc vbe_info_structure
	at vbe_info_structure.signature, db	"VESA"
	iend
	

	;; Storing information
	width					dw 0
	height					dw 0
	bpp						db 0
	vbe_info_segment		dw 0
	vbe_info_offset			dw 0
	mode					dw 0
