#include "../cpu/isr.h"
#include "../drivers/screen.h"
#include "kernel.h"
#include "../libc/string.h"
#include "../programs/texteditor.h"
#include "../util/font.h"
#include "../util/terminal.h"
#include "../drivers/floppy.h"
#include "../cpu/timer.h"
#include "../util/commands.h"
#include "../libc/mem.h"

void generate_fractal(u16 x, u16 y, u16 w, u16 h);

/*void loop()
{
	println("loop just ran");
	}*/

void main() {
	isr_install();
	irq_install();

	setup_video();

	set_background(0, 0, 0);
	clear_screen();

	print_centered(SCN_WIDTH, 8, "_______ _________ _______           _______  _______  _______ ");
	print_centered(SCN_WIDTH, 16,"(  ____ \\\\__   __/(  ____ \\|\\     /|(  ____ \\(  ___  )(  ____ \\");
	print_centered(SCN_WIDTH, 24,"| (    \\/   ) (   | (    \\/| )   ( || (    \\/| (   ) || (    \\/");
	print_centered(SCN_WIDTH, 32, "| (_____    | |   | (__    | |   | || (__    | |   | || (_____ ");
	print_centered(SCN_WIDTH, 40, "(_____  )   | |   |  __)   ( (   ) )|  __)   | |   | |(_____  )");
	print_centered(SCN_WIDTH, 48, "      ) |   | |   | (       \\ \\_/ / | (      | |   | |      ) |");
	print_centered(SCN_WIDTH, 56, "/\\____) |   | |   | (____/\\  \\   /  | (____/\\| (___) |/\\____) |");
	print_centered(SCN_WIDTH, 64, "\\_______)   )_(   (_______/   \\_/   (_______/(_______)\\_______)");
	set_offset(0, 80);

	generate_fractal(SCN_WIDTH - 64 - 8, 8, 64, 64);

	if (init_floppy() != 0)
	{
		println("WARNING: This type of floppy drive is not supported by the system.");
	}

	println("STEVEOS V0.0.3");
	println("WRITTEN BY STEPHEN DOWNWARD");
	println("TYPE \"HELP\" FOR HELP.");
	print("> ");

	register_commands();

	disk_cmd_init();

	while(1)
	{
		//Runs everything that needs it

		flp_main_func();
	}
}

//Mandelbrot Generator
void generate_fractal(u16 x, u16 y, u16 w, u16 h)
{
	for(int i = 0; i < w; i++) {
		for (int j = 0; j < h; j++) {
			double Zr = 0;
			double Zrt = 0; //Temp
			double Zi = 0;

			double Cr = ((double)i/w)*3 - 2;
			double Ci = ((double)j/h)*3 - 1.5;

			int numIterations = 0;
			u8 painted = 0;
			while (numIterations < 255) {
				numIterations++;
				Zrt = Zr * Zr + Cr - (Zi * Zi);
				Zi = 2 * Zi * Zr + Ci;
				Zr = Zrt;
				
				if(Zr * Zr + Zi * Zi > 4) //Divergent
				{
					Color_t p;
					p.r = numIterations * 10;
					p.g = numIterations * 30;
					p.b = numIterations * 50;
					putpixel(x + i, y + j, color_to_pixel(p));
					painted = 1;
					break;
				}
			}
			if (painted == 0) {
				putpixel(x + i, y + j, 0);
			}
		}
	}
}
