#include "terminal.h"
#include "font.h"
#include "../drivers/screen.h"
#include "../libc/string.h"
#include "../drivers/keyboard.h"

int offset_x = 0;
int offset_y = 8; //Higher numbers really means lower on screen

u32 bg;
u32 fg;

void newline() {
	offset_x = 0;
	offset_y += char_height;
	if (offset_y >= SCN_HEIGHT) {
		scroll_up(char_height);
		fillrect(0, SCN_HEIGHT - char_height, bg, SCN_WIDTH, char_height); //Clear last line
		offset_y -= char_height;
	}
}

void print(char *c) {
	int i = 0;
	while (c[i] != 0) {
		if (c[i] >= 97 && c[i] <= 122) { //UPPERCASE because my lowercase font sucks
			c[i] -= 32;
		}
		if (c[i] == '\n') {
			newline();
		} else {
			if (offset_x + 8 > SCN_WIDTH) {
				newline();
			}
			draw_char(offset_x, offset_y, c[i], fg);
			offset_x += 8;
		}
		i++;
	}
}

void println(char *c) {
	print(c);
	newline();
}

void delete_last() {
	offset_x -= 8;
	if (offset_x < 0) {
		offset_x = SCN_WIDTH - 8;
		offset_y -= char_height;
	}
	fillrect(offset_x, offset_y - 11, bg, 8, char_height);
}

void clear_screen() {
	fillrect(0, 0, bg, SCN_WIDTH, SCN_HEIGHT); //Background
	offset_x = 0;
	offset_y = 8;
}

void set_background(int r, int g, int b) {
	Color_t bg_c;
	bg_c.r = r;
	bg_c.g = g;
	bg_c.b = b;
	bg = color_to_pixel(bg_c);

	//Tmp
	Color_t fg_c;
	fg_c.r = 0;
	fg_c.g = 0xcc;
	fg_c.b = 0;
	fg = color_to_pixel(fg_c);
}

void set_offset(int x, int y) {
	offset_x = x;
	offset_y = y;
}

char cmd_history[50][512]; //First element holds key_buffer when we press up
u8 cmd_his_len = 0;
char key_buffer[512];

void (*ext_handler)(char key);
void (*special_handler)(enum SPECIAL key);

void register_keyboard_handler(void (*handler)(char key), void(*special)(enum SPECIAL key))
{
	ext_handler = handler;
	special_handler = special;
}

void remove_keyboard_handler()
{
	ext_handler = 0;
	special_handler = 0;
}

void keyboard_handler(char key)
{
	//Print character
	char printstr[] = {key, 0};
	print(printstr);

	if(!ext_handler)
	{
		append(key_buffer, key);
	}
	else
	{
		ext_handler(key);
	}
}

u8 cmd_his_index = 0;
void goto_cmd_history()
{
	for (int i = 0; i < strlen(key_buffer); i++)
	{
		delete_last();
	}
	strcpy(key_buffer, cmd_history[cmd_his_index]);
	print(cmd_history[cmd_his_index]);
}

void keyboard_handler_special(enum SPECIAL key) {
	//Note: we check for ext_handler, not special_handler, because we don't want the terminal
	//to still act like a terminal when a cmd program is running.
	if(!ext_handler)
	{
		if (key == BACKSPACE)
		{
			if (strlen(key_buffer) > 0)
			{
				backspace(key_buffer);
				delete_last();
			}
		}
		else if (key == ENTER)
		{
			cmd_his_index = 0;
			//Shift the cmd_history array down
			for (int i = 48; i >= 1; i--)
			{
				strcpy(cmd_history[i + 1], cmd_history[i]);
			}
			cmd_his_len++;
			strcpy(cmd_history[1], key_buffer);
			
			newline();
			if(!parse_command(key_buffer))
			{
				println("Command not found.");
			}
			key_buffer[0] = '\0';
			print("> ");
		}
		else if (key == ARW_UP)
		{
			if (cmd_his_index==0)
			{
				strcpy(cmd_history[0], key_buffer);
			}
			if (cmd_his_index < cmd_his_len)
			{
				cmd_his_index++;
				goto_cmd_history();
			}
		}
		else if (key == ARW_DOWN)
		{
			if (cmd_his_index > 0)
			{
				cmd_his_index--;
				goto_cmd_history();
			}
		}
	}
	else if(special_handler)
	{
		special_handler(key);
	}
}

char *commands[255]; //Should be plenty, at least for now.
void (*cmd_handlers[255])(char *);
u8 cmd_index = 0;

void register_command(char *cmd, void (*handler)(char *)) {
	tolowercase(cmd);
	cmd_handlers[cmd_index] = handler;
	commands[cmd_index] = cmd;
	cmd_index++;
}

int parse_command(char *linein) {
	for(int i = 0; i < cmd_index; i++) {
		char cmd[256] = "";
		char args[256] = "";
		strsplit(linein, ' ', cmd, args);
		tolowercase(cmd);
		if(strcmp(cmd, commands[i]) == 0) {
			cmd_handlers[i](args);
			return 1;
		}
	}
	return 0;
}
